#!/usr/bin/python

import socket, psutil, os

ADDR = './local_socket'
serv = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)

if os.path.exists(ADDR):
  print('socket exists. Removing it')
  os.remove(ADDR)

serv.bind(ADDR)
serv.listen(1)

print 'listening ...'

while True:
  conn, addr = serv.accept()
  print 'client connected ... ', addr
  cpu_percent = str(psutil.cpu_percent())
  conn.send(cpu_percent)
  print('written ' + cpu_percent)
  conn.close()
  print('client disconnected')



#!/usr/bin/python

import psutil, serial, time
from subprocess import Popen

Popen(["socat", "-d", "-d", "PTY,raw,echo=0,link=./serial_in", "PTY,raw,echo=0,link=./serial_out"])
time.sleep(1)

# rtscts=True,dsrdtr=True
s = serial.Serial("./serial_in", 9600)
print(s)

while True:
  cpu_percent = str(psutil.cpu_percent(interval=2.))
  s.write(cpu_percent)
  print('written ' + cpu_percent)



import sys
import ltspice
import os

if __name__ == "__main__":
    if len(sys.argv) < 2:
        raise Exception("Pass the filename as second argument!")
    filenameWithPath = sys.argv[1]

    (path, filename) = os.path.split(filenameWithPath)

    namespace = filename.split(".")[0]

    l = ltspice.Ltspice(filenameWithPath)
    l.parse()

    isComplex = False
    if "complex" in l.flags:
        isComplex = True

    data_str = ""
    size = len(l.x_raw)
    for r in range(size):
        line = ""
        for index, v in enumerate(l.variables):
            if v == "time":
                s = l.get_time()
            elif v == "frequency":
                s = l.get_frequency()
            else:
                s = l.get_data(v)
            line += f'{s[r].real:{1}.{15}e}'
            if isComplex:
                line += ","
                line += f'{s[r].imag:{1}.{15}e}'

            if index < len(l.variables) - 1:
                line += ","
        data_str += f"{line}\n"

    #data_str = f"QVector<QStringList> refData = {{{data_str}}};"
    #filename_string = f'const QString filename = "{filename}";\n'


    #data_str = f"namespace {namespace} {{\n\n{filename_string}\n{data_str}\n}}"



    with open(filenameWithPath + ".refdata", "w") as f:
        f.write(data_str)




import sys
import struct

if __name__ == "__main__":
    if len(sys.argv) < 2:
        raise Exception("Pass the filename as second argument!")
    filename = sys.argv[1]

    with open(filename, "rb") as f:

        number_variables = 0
        number_points = 0
        line = f.readline()
        flags = b""
        while b"Binary:" not in line:
            if b"No. Variables: " in line:
                number_variables = int(line.split(b"No. Variables: ")[1])
            if b"No. Points: " in line:
                number_points = int(line.split(b"No. Points: ")[1])
            if b"Flags: " in line:
                flags = line.split(b"Flags: ")[1]
            line = f.readline()

        isComplex = b"complex" in flags
        #d = bytearray(f.read())
        data_str = ""
        for r in range(number_points):
            line = ''
            for c in range(number_variables):
                value = struct.unpack('d', f.read(8))[0]  # every data is a double (8byte)
                line += f'"{value:{1}.{15}e}"'
                if isComplex:
                    line += ", "
                    value = struct.unpack('d', f.read(8))[0]  # every data is a double (8byte)
                    line += f'"{value:{1}.{15}e}"'
                if c < number_variables - 1:
                    line += ", "


            line = f"{{{line}}}"
            if r < number_points - 1:
                line += ",\n"
            data_str += line
        data_str = f"QVector<QStringList> refData = {{{data_str}}};"

        with open(filename + ".data", "w") as f:
            f.write(data_str)




#!/usr/bin/python3

import socket, psutil, subprocess
import sys, errno

#HOST = '192.168.178.26'
HOST = 'localhost'
PORT = 1030
ADDR = (HOST,PORT)
serv = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

serv.bind(ADDR)
serv.listen(1)

# download pinpoit from https://github.com/osmhpi/pinpoint
# use pinpoint -l to get the list of counters and aliases for them,
# adjust the command below - e.g. use CPU,RAM only if not counter is available for the GPU, etc.
pinpoint_cmd = "pinpoint -e CPU,GPU,RAM -i 200 -c sleep 0.1"

print ('listening ...')

while True:
  conn, addr = serv.accept()
  print('client connected ... ', addr)
  cpu_percent = str(psutil.cpu_percent())
  pinpoint_output = subprocess.check_output(pinpoint_cmd, shell=True).decode("utf-8")
  result = str(cpu_percent) + ',' + pinpoint_output
  
  try:
    conn.send(result.encode())
    print('written ' + result)
  except (BrokenPipeError, IOError):
    pass

  conn.close()
  print('client disconnected')



#!/usr/bin/python

from socket import *
import psutil

HOST = 'localhost'
PORT = 1027
ADDR = (HOST,PORT)

serv = socket(AF_INET, SOCK_DGRAM)
serv.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
while True:
  cpu_percent = str(psutil.cpu_percent(interval=0.5))
  serv.sendto(cpu_percent.encode(), ADDR)
  print('written ' + cpu_percent, ADDR)



#!/usr/bin/python

import time
from scipy.stats import norm
import numpy as np

LIVE_DATA = True
LIVE_DATA_DEBUG = True

fileName = "out.txt" # name of the output file
sleepInterval = 1 # sleep interval in seconds
itersTotal = 100000 # total number of iterations to compute
iters = 1000 # number of iterations done before the results are written out to the file
pathes = 2 # Number of pathes to compute
delta = 0.25
dt = 0.1


# trajectories
x = np.zeros(pathes)

# Iterate to compute the steps of the Brownian motion.
i = 0
out = ""
for k in range(itersTotal):
	if out != "":
		out += "\n"

	out += str(k*dt)
	for p in range(pathes):
		x[p] += norm.rvs(scale=delta**2*dt)
		out += "\t" + str(x[p])

	i += 1

	if LIVE_DATA and i >= iters:
		if LIVE_DATA_DEBUG:
			print (out)
		# generate header
		outh = "t"
		for p in range(pathes):
			outh += "\tx" + str(p+1)

		datafile = open(fileName, "w")

		datafile.write(outh)
		datafile.write('\n')
		datafile.write(out)
		datafile.close()

		out = ""
		i = 0
		time.sleep(sleepInterval)

if LIVE_DATA and out != "":
	datafile = open(fileName, "a")
	datafile.write(out)
	datafile.close()
	if LIVE_DATA_DEBUG:
		print (out)

if not LIVE_DATA:
	datafile = open(fileName, "a")
	datafile.write(out)



#!/usr/bin/python

from socket import *
import psutil

HOST = 'localhost'
PORT = 1027
ADDR = (HOST,PORT)
client = socket(AF_INET, SOCK_DGRAM)
client.bind(ADDR)
while True:
  data, addr = client.recvfrom(1024)
  print('read ' + data, addr)





#!/usr/bin/env python3
import numpy as np
BSIZE_SP = 512 # Max size of a line of data; we don't want to read the
               # whole file to find a line, in case file does not have
               # expected structure.
MDATA_LIST = [b'title', b'date', b'plotname', b'flags', b'no. variables',
              b'no. points', b'dimensions', b'command', b'option']

def rawread(fname: str):
    """Read ngspice binary raw files. Return tuple of the data, and the
    plot metadata. The dtype of the data contains field names. This is
    not very robust yet, and only supports ngspice.
    >>> darr, mdata = rawread('test.py')
    >>> darr.dtype.names
    >>> plot(np.real(darr['frequency']), np.abs(darr['v(out)']))
    """
    # Example header of raw file
    # Title: rc band pass example circuit
    # Date: Sun Feb 21 11:29:14  2016
    # Plotname: AC Analysis
    # Flags: complex
    # No. Variables: 3
    # No. Points: 41
    # Variables:
    #         0       frequency       frequency       grid=3
    #         1       v(out)  voltage
    #         2       v(in)   voltage
    # Binary:
    fp = open(fname, 'rb')
    plot = {}
    count = 0
    arrs = []
    plots = []
    while (True):
        try:
            mdata = fp.readline(BSIZE_SP).split(b':', maxsplit=1)
        except:
            raise
        if len(mdata) == 2:
            if mdata[0].lower() in MDATA_LIST:
                plot[mdata[0].lower()] = mdata[1].strip()
            if mdata[0].lower() == b'variables':
                nvars = int(plot[b'no. variables'])
                npoints = int(plot[b'no. points'])
                plot['varnames'] = []
                plot['varunits'] = []
                for varn in range(nvars):
                    varspec = (fp.readline(BSIZE_SP).strip()
                               .decode('ascii').split())
                    assert(varn == int(varspec[0]))
                    plot['varnames'].append(varspec[1])
                    plot['varunits'].append(varspec[2])
            if mdata[0].lower() == b'binary':
                rowdtype = np.dtype({'names': plot['varnames'],
                                     'formats': [np.complex_ if b'complex'
                                                 in plot[b'flags']
                                                 else np.float_]*nvars})
                # We should have all the metadata by now
                arrs.append(np.fromfile(fp, dtype=rowdtype, count=npoints))
                plots.append(plot)
                fp.readline() # Read to the end of line
        else:
            break
    return (arrs, plots)

if __name__ == '__main__':
    arrs, plots = rawread('data/ac_binary.raw')
    print(arrs)



#!/usr/bin/python

import serial

#import serial.tools.list_ports
#print list(serial.tools.list_ports.comports())

s = serial.Serial("./serial_out", 9600,timeout=2)
print(s)

while True:
  x = s.read(3)
  print('read ' + x)



#!/usr/bin/python3

import socket, psutil

HOST = 'localhost'
PORT = 1027
ADDR = (HOST,PORT)
serv = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
serv.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

serv.bind(ADDR)
serv.listen(1)

print ('listening ...')

while True:
  conn, addr = serv.accept()
  print('client connected ... ', addr)
  #message = str(psutil.cpu_percent())
  # multiple values
  message = str(psutil.cpu_percent()) + " " + str(psutil.boot_time()) + " " + str(psutil.cpu_count())
  conn.send(message.encode())
  print('written ' + message)
  conn.close()
  print('client disconnected')



#!/usr/bin/python

import psutil

while True:
    cpu_percents = str(psutil.cpu_percent(interval=0.5, percpu=True))
    line = str(cpu_percents)[1:-1]
    print line

    datafile = open("cpuData.txt", "a")
    datafile.write(line + "\n")
    datafile.close()
